{$APPTYPE CONSOLE}
program delphiSmart;
uses SysUtils, Classes, Math;
const
  MAX_SIZE = 100;
  PLAYER_NAME = 'delphi random';
type
  TPoint = record
    x: Integer;
    y: Integer;
  end;
  
  TCell = record
    owner: Integer;
    isVeteran: Boolean;
    isConnected: Boolean;
  end;

  TMap = array[0..MAX_SIZE, 0..MAX_SIZE] of TCell;

var
  me: Integer;
  size: Integer;
  map: TMap;
  legionsCount: Integer;

procedure InputData();
var
  x, y: Integer;
  c1, c2: char;
begin
  legionsCount := 5;
  Readln(me);
  Readln(size);
  Readln;
  Readln;
  for y := 0 to size-1 do
  begin
    for x := 0 to size - 1 do
    begin
      Read(c1);
      Read(c2);
      if (c1 <> '.') then
      begin
        map[x][y].isVeteran := c1 in ['V', 'v'];
        map[x][y].isConnected := c1 in ['N', 'V'];
        map[x][y].owner := StrToInt(c2);
        Read(c2);
      end
      else
      begin
        map[x][y].isVeteran := false;
        map[x][y].isConnected := false;
        map[x][y].owner := -1;
      end
    end;
    Readln;
  end;
end;

function InMap(x, y: Integer): Boolean;
begin
  result := (x >= 0) and (x < size) and (y >= 0) and (y < size);
end;

function IsMyConnected(x, y: Integer): Boolean;
begin
  if InMap(x, y) then
    result := (map[x][y].owner = me) and map[x][y].isConnected
  else
    result := false;
end;

function IsMyNovices(x, y: Integer): Boolean;
begin
  if InMap(x, y) then
    result := (map[x][y].owner = me) and not map[x][y].isVeteran
  else
    result := false;
end;

function CanMoveTo(x, y: Integer): Boolean;
begin
  if map[x][y].owner = me then
    result := false
  else if (map[x][y].owner >= 0) and (legionsCount < 2) then
    result := false
  else if (map[x][y].owner >= 0) and (map[x][y].isVeteran) and (legionsCount < 3) then
    result := false
  else
    result := IsMyConnected(x-1, y) or IsMyConnected(x+1, y)
           or IsMyConnected(x, y-1) or IsMyConnected(x, y+1);
end;

type
  TAnalyserFun = function(x, y: Integer): Boolean;


function CountNear(x, y: Integer; func: TAnalyserFun): Integer;
begin
  result := 0;
  if InMap(x-1, y) and func(x-1, y) then Inc(result);
  if InMap(x+1, y) and func(x+1, y) then Inc(result);
  if InMap(x, y-1) and func(x, y-1) then Inc(result);
  if InMap(x, y+1) and func(x, y+1) then Inc(result);
end;

function HasNear(x, y: Integer; func: TAnalyserFun): Boolean;
begin
  result :=
     InMap(x-1, y) and func(x-1, y)
  or InMap(x+1, y) and func(x+1, y)
  or InMap(x, y-1) and func(x, y-1)
  or InMap(x, y+1) and func(x, y+1);
end;

function EnemyNovice(x, y: Integer): Boolean;
begin
  result := (map[x][y].owner <> me) and (map[x][y].owner <> -1) and not map[x][y].isVeteran;
end;

function EnemyVeteran(x, y: Integer): Boolean;
begin
  result := (map[x][y].owner <> me) and (map[x][y].owner <> -1) and map[x][y].isVeteran;
end;

function MyNovice(x, y: Integer): Boolean;
begin
  result := (map[x][y].owner = me) and not map[x][y].isVeteran;
end;

function MyVeteran(x, y: Integer): Boolean;
begin
  result := (map[x][y].owner = me) and map[x][y].isVeteran;
end;

function My(x, y: Integer): Boolean;
begin
  result := (map[x][y].owner = me);
end;

function Any(x, y: Integer): Boolean;
begin
  result := true;
end;

function GetEmptyCost(x, y: Integer): Extended;
begin
  if HasNear(x, y, EnemyNovice) then
    if legionsCount >= 3 then result := 4/3 + 0.001
    else result := 0.5
  else if HasNear(x, y, EnemyVeteran) then
    if legionsCount >= 4 then result := 4/3 - 0.1
    else result := 0.5
  else
    result := 1 - 0.01*(4 - CountNear(x, y, Any)) + 0.01 * CountNear(x, y, My);
end;

function MoveCost(x, y: Integer): Extended;
begin
  if map[x][y].owner = me then result := 0
  else if map[x][y].owner = -1 then result := GetEmptyCost(x, y)
  else if map[x][y].isVeteran then result := 4/3
  else result := 3/2;
end;

procedure MoveTo(x, y: Integer);
var
  c: Integer;
begin
  if (map[x][y].owner = me) or (map[x][y].owner = -1) then c := 1
  else if map[x][y].isVeteran then c := 3
  else c := 2;
  if (legionsCount >= c) then
  begin
    map[x][y].owner := me;
    map[x][y].isVeteran := c > 1;
    map[x][y].isConnected := true;
  end;
  Writeln(x, ' ', y, ' ', Min(legionsCount, c));
  Dec(legionsCount, c);
end;

procedure Move();
var
  x, y: Integer;
  bx, by: array[0..MAX_SIZE] of Integer;
  ind, bcount: Integer;
  cost, bcost: Extended;
begin
  while (legionsCount > 0) do
  begin
    bcost := -1;
    bcount := 0;
    for y := 0 to size-1 do
      for x := 0 to size-1 do
      begin
        cost := MoveCost(x, y);
        if CanMoveTo(x, y) then
        begin
          if cost = bcost then
          begin
            bx[bcount] := x;
            by[bcount] := y;
            Inc(bcount);
          end
          else if cost > bcost then
          begin
            bx[0] := x;
            by[0] := y;
            bcount := 1;
            bcost := cost;
          end;
        end;
      end;
      if bcount > 0 then
      begin
        ind := Random(bcount);
        MoveTo(bx[ind], by[ind]);
      end
      else Exit;
    end;
end;

begin
  Randomize;
  if SeekEOF then
    Writeln(PLAYER_NAME)
  else
  begin
    InputData();
    Move();
  end;
end.
